// PlotData.cpp: implementation of the CPlotData class.
//
//////////////////////////////////////////////////////////////////////
// Copyright 2000.    Moe Wheatley AE4JY  <ae4jy@mindspring.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////

#include "stdafx.h"
#include "ambetest.h"
#include "PlotData.h"
#include "ambetestview.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CPlotData::CPlotData()
{
	m_pDoc = NULL;

//allocate memory space
	m_pDispBuf = new INT[DISPBUF_SIZE+10];
	ASSERT( m_pDispBuf );
	for(int i=0; i<DISPBUF_SIZE; i++)
		m_pDispBuf[i] = 0;
//allocate FFT object
	m_pPenR = new CPen;		// create some colored pens for drawing
	m_pPenG = new CPen;
	m_pPenY = new CPen;
	m_pPenW = new CPen;
	m_pPenC = new CPen;
	m_pPenB = new CPen;
	m_pPenR->CreatePen( PS_SOLID,0, RGB(255,0,0) );
	m_pPenG->CreatePen( PS_SOLID,0, RGB(0,255,0) );
	m_pPenY->CreatePen( PS_DOT,0, RGB(255,255,0) );
	m_pPenW->CreatePen( PS_SOLID,0, RGB(128,128,128) );
	m_pPenC->CreatePen( PS_SOLID,0, RGB(0,255,255) );
	m_pPenB->CreatePen( PS_SOLID,0, RGB(0,0,255) );
}

CPlotData::~CPlotData()
{
	m_pPenR->DeleteObject(); delete m_pPenR;
	m_pPenG->DeleteObject(); delete m_pPenG;
	m_pPenY->DeleteObject(); delete m_pPenY;
	m_pPenW->DeleteObject(); delete m_pPenW;
	m_pPenC->DeleteObject(); delete m_pPenC;
	m_pPenB->DeleteObject(); delete m_pPenB;
	if(m_pDispBuf)
	{
		delete m_pDispBuf;
		m_pDispBuf = NULL;
	}
}


//////////////////////////////////////////////////////////////////////
//  Called from the Windows view class when the window has changed
//      and needs to update the screen coordinates.
//////////////////////////////////////////////////////////////////////
void CPlotData::OnDrawPlot(HWND hWnd, CFormView* pView, CAmbeTestDoc* pDoc)
{
CRect rScrn;
	m_pDoc = pDoc;
	m_hViewWnd = hWnd;

// Get pixel size of Input plot area bitmap
	ASSERT_VALID( pView );
	CWnd* pWnd = pView->GetDlgItem( IDC_INPLOT );
	ASSERT_VALID( pWnd);
	pWnd->GetWindowRect( m_rInPlot );
	pView->ScreenToClient(m_rInPlot);
	m_rInPlotsize.left = 0;
	m_rInPlotsize.top = 0;
	m_rInPlotsize.right = m_rInPlot.Width();
	m_rInPlotsize.bottom = m_rInPlot.Height();
// Get pixel size of Output plot area bitmap
	pWnd = pView->GetDlgItem( IDC_OUTPLOT );
	ASSERT_VALID( pWnd);
	pWnd->GetWindowRect( m_rOutPlot );
	pView->ScreenToClient(m_rOutPlot);
	m_rOutPlotsize.left = 0;
	m_rOutPlotsize.top = 0;
	m_rOutPlotsize.right = m_rOutPlot.Width();
	m_rOutPlotsize.bottom = m_rOutPlot.Height();

}


//////////////////////////////////////////////////////////////////////
//  Called to clear plot surface, draw cursor, and call specific plot
//         routine.
//////////////////////////////////////////////////////////////////////
void CPlotData::DrawPlot()
{
// erases plot surface
// plots data points
CDC MemDC;
CPen* pOldPen;
CClientDC ClientDC(CWnd::FromHandle(m_hViewWnd) );
	if( m_bmInPlot.m_hObject != NULL )	// delete old bitmap objects
		m_bmInPlot.DeleteObject();
	if( m_bmOutPlot.m_hObject != NULL )	// delete old bitmap objects
		m_bmOutPlot.DeleteObject();
//create a new bitmap with current screen DC settings
	m_bmInPlot.CreateCompatibleBitmap( &ClientDC, m_rInPlot.Width(),
					 m_rInPlot.Height() );
	m_bmOutPlot.CreateCompatibleBitmap( &ClientDC, m_rOutPlot.Width(),
					 m_rOutPlot.Height() );
//create memory DC pointer to the plot bitmap
	MemDC.CreateCompatibleDC( &ClientDC );

//==========Plot Output data============
CBitmap* pbmOld = MemDC.SelectObject( &m_bmOutPlot );
// set bitmap background to black using slightly faster ExtTextOut function
	MemDC.SetBkColor( RGB(0, 0, 0) );
	MemDC.ExtTextOut( 0, 0,ETO_OPAQUE, &m_rOutPlotsize,NULL,0,NULL);
	pOldPen = MemDC.SelectObject(m_pPenG);		//save original pen.
	PlotOutData( &MemDC );
// Copy memory bitmap into screen device context for flicker free update
	ClientDC.BitBlt( m_rOutPlot.left, m_rOutPlot.top,
					m_rOutPlotsize.right,
					m_rOutPlotsize.bottom, &MemDC,
					0,0,SRCCOPY);
//==========Plot Input data ============
	MemDC.SelectObject( &m_bmInPlot );
// set bitmap background to black using slightly faster ExtTextOut function
	MemDC.SetBkColor( RGB(0, 0, 0) );
	MemDC.ExtTextOut( 0, 0,ETO_OPAQUE, &m_rInPlotsize,NULL,0,NULL);

	MemDC.SelectObject(m_pPenR);
	PlotInData( &MemDC );
	// Copy memory bitmap into screen device context for flicker free update
	ClientDC.BitBlt( m_rInPlot.left, m_rInPlot.top,
					m_rInPlotsize.right,
					m_rInPlotsize.bottom, &MemDC,
					0,0,SRCCOPY);

/////////////////////////	
	MemDC.SelectObject(pOldPen);	//deselect pens and bitmap
	MemDC.SelectObject( pbmOld );	// from memory DC
}



//////////////////////////////////////////////////////////////////////
// Plots the input data
//////////////////////////////////////////////////////////////////////
void CPlotData::PlotInData( CDC* pDC)
{
INT x,y,k1;
	k1 = m_rInPlotsize.bottom/2;		//y range
	pDC->MoveTo( 0, k1);			//move to first position
	m_pDoc->GetInSoundData( m_pDispBuf );
	for( x=1; x<m_rInPlotsize.right; x++ )
	{
		y = k1 + (k1*m_pDispBuf[x])/32767;
		pDC->LineTo( x, y );	//draw to the next pt.
	}
}



//////////////////////////////////////////////////////////////////////
// Plots the output data
//////////////////////////////////////////////////////////////////////
void CPlotData::PlotOutData( CDC* pDC)
{
INT x,y,k1;
	k1 = m_rOutPlotsize.bottom/2;		//center
	pDC->MoveTo( 0, k1);			//move to first position
	m_pDoc->GetOutSoundData( m_pDispBuf );
	for( x=1; x<m_rOutPlotsize.right; x++ )
	{
		y = k1 + (k1*m_pDispBuf[x])/32767;
		pDC->LineTo( x, y );	//draw to the next pt.
	}
}



