// SerialPort.cpp: implementation of the CSerialPort class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AmbeTest.h"
#include "SerialPort.h"
#include "voicedata.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MSGSTATE_HDR1 0		//ASCP partial msg parsing states
#define MSGSTATE_HDR2 1
#define MSGSTATE_DATA 2


// define some offsets into msg buffer
#define MSG_SIZE 0			//location of msg size and type
#define MSG_TYPE 1
#define MSG_CITEM 2			//location of 16 bit CItem value
#define MSG_PARAM 4			//start of CItem parameters
#define MSG_DATAADR 2		//start of Data in Data msg


#define MSG_TIMEOUT 25

#define RECORD_BUF_SIZE 1000000

const unsigned short GROUP_TO_RATECODE[4][5] = 
{
	0x0028, 0x0000, 0x0000, 0x0000, 0x6428,		//2000
	0x1030, 0x4000, 0x0000, 0x0000, 0x0048,		//2400
	0x5360, 0x0000, 0x0000, 0x0000, 0x6C60,		//4800
	0x7FC0, 0x0000, 0x0000, 0x0000, 0x69C0		//9600
};

const unsigned short GROUP_TO_FRAMEBYTES[4] = 
{
	5, 9, 12, 24
};


int g_Debug = 0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSerialPort::CSerialPort()
{
	m_ParseState = MSGSTATE_HDR1;
	m_RxMsgLength = 0;
	InitializeCriticalSection(&m_CriticalSection);
	m_pRecordBuf = NULL;
	m_RecordedSize = 0;
	m_RecorderPos = -1;
	m_Recording = FALSE;
	m_DTMFActive = FALSE;
	m_VoiceDetect = FALSE;
	m_VoiceRate = 0;
	m_SerialNumber = "";
	m_pRecordBuf = new unsigned char[RECORD_BUF_SIZE];
	for(int i=0; i<RECORD_BUF_SIZE ;i++)
		m_pRecordBuf[i] = 0;
	g_Debug = 0;

}

CSerialPort::~CSerialPort()
{
	DeleteCriticalSection(&m_CriticalSection);
	if(m_pRecordBuf)
		delete m_pRecordBuf;
}

LONG CSerialPort::StartListener(int VoiceRate)
{
int i;
	m_VoiceRate = VoiceRate;
	m_DTMFActive = FALSE;
	m_SaveActive = FALSE;
	m_SaveArmed = FALSE;
	m_VoiceDetect = FALSE;
	m_ParseState = MSGSTATE_HDR1;
	m_RxMsgLength = 0;
	for(i=0; i<24; i++)
		m_InFrame.All[i] = 0;
	m_InFrame.fld.Hdr = AMBE2020_FRAME_SYNC;
	m_InFrame.fld.CW1 = 0x0000;

	m_InFrame.fld.Rate0 = GROUP_TO_RATECODE[m_VoiceRate][0];
	m_InFrame.fld.Rate1 = GROUP_TO_RATECODE[m_VoiceRate][1];
	m_InFrame.fld.Rate2 = GROUP_TO_RATECODE[m_VoiceRate][2];
	m_InFrame.fld.Rate3 = GROUP_TO_RATECODE[m_VoiceRate][3];
	m_InFrame.fld.Rate4 = GROUP_TO_RATECODE[m_VoiceRate][4];
	
	m_InFrame.fld.DTMFCtrl = 0x00FF;
		
//	m_InFrame.fld.CW2 = 0xF004;		//Encoder(RF Tx mode) enable
	m_InFrame.fld.CW2 = 0x8020;		//Decoder(RF Rx mode) enable

	m_NumFrameBytes = GROUP_TO_FRAMEBYTES[VoiceRate];
	for(i=0; i<24; i++)
		m_InFrame.fld.ChData[i] = 0x00;
	CalcBitErrors();
	m_RxBitErrors = 0;
	return CSerialEx::StartListener();
}


void CSerialPort::OnEvent(EEvent eEvent, EError eError)
{
DWORD dwBytesRead = 0;
unsigned char TBuf[8192];
	if( CSerial::EEventRecv == eEvent)
	{
		m_MsgActive = TRUE;
		m_MsgTimer = MSG_TIMEOUT;
		EnterCriticalSection(&m_CriticalSection);
		Read(TBuf, sizeof(TBuf), &dwBytesRead);
		LeaveCriticalSection(&m_CriticalSection);
//TRACE("\r\n%d", dwBytesRead);
		if (dwBytesRead > 0)
		{
			for(DWORD i=0; i<dwBytesRead; i++)
				AssembleMsg( TBuf[i] );
		}
	}
	else if( CSerial::EEventError == eEvent)
	{
		EnterCriticalSection(&m_CriticalSection);
		Read(TBuf, sizeof(TBuf), &dwBytesRead);
		LeaveCriticalSection(&m_CriticalSection);
	}
}

//////////////////////////////////////////////////////////////////////
//called to Assemble received bytes into an ASCP message format
//////////////////////////////////////////////////////////////////////
void CSerialPort::AssembleMsg(unsigned __int8 ch)
{
	switch(m_ParseState)	//Simple state machine to get generic ASCP msg
	{
		case MSGSTATE_HDR1:		//get first header byte of ASCp msg
			m_RxASCPmsg.base.h0= ch;
			m_ParseState = MSGSTATE_HDR2;	//go to get second header byte state
			break;
		case MSGSTATE_HDR2:	//here for second byte of header
			m_RxASCPmsg.base.h1 = ch;
			m_RxMsgLength = m_RxASCPmsg.citem.header&LENGTH_MASK;
			m_RxMsgIndex = 2;
			if(2 == m_RxMsgLength)	//if msg has no parameters then we are done
			{
				m_ParseState = MSGSTATE_HDR1;	//go back to first state
				ParseASCPmsg(&m_RxASCPmsg);	//call ASCP message parsing routine
			}
			else	//there are data bytes to fetch
			{
				if( 0 == m_RxMsgLength)
					m_RxMsgLength = 8192+2;	//handle special case of 8192 byte data message
				m_ParseState = MSGSTATE_DATA;	//go to data byte reading state
			}
//TRACE("\r\n%d", m_RxMsgLength);
			break;
		case MSGSTATE_DATA:	//try to read the rest of the message or to end of buffer
			m_RxASCPmsg.buf[m_RxMsgIndex++] = ch;
			if(m_RxMsgIndex >= m_RxMsgLength)	//end of the msg?
			{
				m_ParseState = MSGSTATE_HDR1;	//go back to first stage
				ParseASCPmsg(&m_RxASCPmsg);	//call ASCP message parsing routine
				m_MsgActive = FALSE;
			}
			break;
		default:
			m_MsgActive = FALSE;
			m_ParseState = MSGSTATE_HDR1;	//reset parser to first stage
			break;
	} //end switch statement
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    -------------------------                              */
/*                   | P a r s e A S C P m s g |                             */
/*                    -------------------------                              */
/*    This routine is called to parse a new message from the Host.           */
/* INPUTS: The Msg Buffer 'UsartRxMsg' containing the new msg from the Host  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CSerialPort::ParseASCPmsg(ASCPstruct* pMsg)
{
int i;
UINT mType;
UINT mCitem;
UINT mLength;
	mType = pMsg->base.h1 & TYPE_MASK;	//strip out just type field
	mCitem = pMsg->citem.CItem;
	mLength = pMsg->citem.header & LENGTH_MASK;
	if( mType == TYPE_TARG_RESP_CITEM )
	{	// Is a response message from target
		switch( mCitem )
		{
			case CI_GENERAL_INTERFACE_NAME:
				break;
			case CI_GENERAL_INTERFACE_SERIALNUM:
				m_SerialNumber = (char*)pMsg->citem.parms;
				break;
			case CI_GENERAL_STATUS_CODE:
				break;
			case CI_RUN_STATE:
				break;
			default:
				break;
		}
	}
	else if( mType == TYPE_TARG_UNSOLICITED_CITEM )
	{	// Is an unsolicited message from target giving a control item value
	}
	else if( mType == TYPE_TARG_DATA_ITEM0 )
	{	//is a codec uncompressed 8000sps 16 bit data packet from the Dongle
		//send dongle codec data to soundcard output 
		//send soundcard input to dongle codec data 
		g_SoundCard.PutSoundOutData( (short*)pMsg->Ditem16.parms );
		if(m_SaveActive )
			m_Wave.OutWrite( (short*)pMsg->Ditem16.parms, 160);

		g_SoundCard.GetSoundInData( (short*)pMsg->Ditem16.parms );
		SendASCPMsg(pMsg);
	}
	else if( mType == TYPE_TARG_DATA_ITEM1 )
	{	//is a compressed data frame from the Dongle
		for(i=0; i<24; i++)	//put incoming compressed data into OutFrame structure
			m_OutFrame.All[i] = pMsg->Ditem16.parms[i];
		if( !(m_OutFrame.fld.CW1 & 0x0009) )
			m_VoiceDetect = TRUE;
		else
			m_VoiceDetect = FALSE;
		if( m_Recording )	//if recording to memory
		{
			m_InFrame.fld.CW2 = 0x0000;		//disable volume during record
			for(i=0; i<m_NumFrameBytes; i++)
				m_pRecordBuf[m_RecorderPos++] = pMsg->Ditem8.parms[i+24];
			if( m_RecorderPos >= RECORD_BUF_SIZE-100 )
			{
				SetRecordingState(FALSE);
			}
		}
		else
		{
			m_RxBitErrors = m_OutFrame.fld.BitErrors;

			if(m_DTMFActive)
			{
				for(i=0; i<24; i++)
					m_InFrame.fld.ChData[i] = 0x00;
				m_InFrame.fld.CW2 = 0x8000;		//Decoder(Playback mode) enable
			}
			else if(m_RecorderPos>=0)	//playing back record buffer
			{
				m_InFrame.fld.CW2 = 0x8020;		//Decoder(Playback mode) enable
				if(m_SaveArmed)
					m_SaveActive = TRUE;
				CalcBitErrors();
				for(i=0; i<m_NumFrameBytes; i++)
					m_InFrame.fld.ChData[i] = m_pRecordBuf[m_RecorderPos++] ^ m_BitErrorMask[i];
				if(m_RecorderPos >= m_RecordedSize)
				{
					m_RecorderPos = -1;
					for(i=0; i<24; i++)
						m_InFrame.fld.ChData[i] = 0xFF;
					m_InFrame.fld.CW2 = 0x8020;		//turn on volume
					if(m_SaveArmed)
					{
						m_SaveActive = FALSE;
						m_SaveArmed = FALSE;
						m_Wave.OutClose();	//close wave file if open
					}
				}
			}
			else	//idle mode
			{
//				m_InFrame.fld.DTMFCtrl = 0xF60A;
				m_InFrame.fld.CW2 = 0x8020;		//turn on volume for echo mode
				CalcBitErrors();
				for(i=0; i<24; i++)
//					m_InFrame.fld.ChData[i] = 0xFF;
					m_InFrame.fld.ChData[i] = m_OutFrame.fld.ChData[i] ^ m_BitErrorMask[i];
			}
		}
		for(i=0; i<24; i++)
			pMsg->Ditem16.parms[i] = m_InFrame.All[i];
		SendASCPMsg(pMsg);
	}
}

//////////////////////////////////////////////////////////////////////
//  Called send an ASCP formated message to the target
// returns TRUE if gets response
//////////////////////////////////////////////////////////////////////
BOOL CSerialPort::SendASCPMsg( ASCPstruct* pASCPmsg)
{
LONG lLastError = ERROR_SUCCESS;
DWORD txbytes;
int timeout = 100;	//xmit timeout ~1 second
DWORD bytesremaining = pASCPmsg->citem.header&LENGTH_MASK;
	if(IsOpen() )
	{
		while(bytesremaining > 0)
		{
			EnterCriticalSection(&m_CriticalSection);
			lLastError = Write((LPVOID)pASCPmsg, bytesremaining, &txbytes);
			LeaveCriticalSection(&m_CriticalSection);
			if (lLastError != ERROR_SUCCESS)
				return FALSE;
			bytesremaining -= txbytes;
			if(bytesremaining>0)	//if driver didn't send them all
			{
				Sleep(100);			//wait
				if(timeout-- <= 0)	//bail if taking too long
					return FALSE;
			}
		}
	}
	else
	{
		return FALSE;
	}
	return TRUE;
}

void CSerialPort::SetTouchTone( BOOL On, char digit )
{
//TRACE("\r\n %4.4X",m_OutFrame.fld.CW1);
	if( On != m_DTMFActive)
	{
		m_DTMFActive = On;
		m_InFrame.fld.DTMFCtrl = 0x00FF;	//disable DTMF
		m_InFrame.fld.CW2 = 0x0020;		//turn off volume
	}
	if(m_DTMFActive)
	{
		switch(digit)
		{
			case '0':
				m_InFrame.fld.DTMFCtrl = 0xF687;
				break;
			case '1':
				m_InFrame.fld.DTMFCtrl = 0xF680;
				break;
			case '2':
				m_InFrame.fld.DTMFCtrl = 0xF684;
				break;
			case '3':
				m_InFrame.fld.DTMFCtrl = 0xF688;
				break;
			case '4':
				m_InFrame.fld.DTMFCtrl = 0xF681;
				break;
			case '5':
				m_InFrame.fld.DTMFCtrl = 0xF685;
				break;
			case '6':
				m_InFrame.fld.DTMFCtrl = 0xF689;
				break;
			case '7':
				m_InFrame.fld.DTMFCtrl = 0xF682;
				break;
			case '8':
				m_InFrame.fld.DTMFCtrl = 0xF686;
				break;
			case '9':
				m_InFrame.fld.DTMFCtrl = 0xF68A;
				break;
			case '*':
				m_InFrame.fld.DTMFCtrl = 0xF683;
				break;
			case '#':
				m_InFrame.fld.DTMFCtrl = 0xF688;
				break;
			default:
				m_InFrame.fld.DTMFCtrl = 0xC480;	//set DTMF volume to -60dB
				break;
		}
	}
}

void CSerialPort::SetRecordingState(BOOL On)
{
	if(On)
	{
		m_RecordedSize = 0;
		m_RecorderPos = 0;
		m_Recording = TRUE;
	}
	else
	{
		m_Recording = FALSE;
		m_RecordedSize = m_RecorderPos;
		m_RecorderPos = 0;
//	SaveToFile();
	}
}

int CSerialPort::GetProgress()
{
	if(m_RecorderPos>=0)
	{
		if(m_RecordedSize>0)
			return ( (m_RecorderPos*100)/m_RecordedSize ); 
		else
			return ( (m_RecorderPos*100)/RECORD_BUF_SIZE ); 
	}
	return 0; 
}

void CSerialPort::CannedVoice()
{
int i;
	switch(m_VoiceRate)
	{
		case 0:
			m_RecordedSize = sizeof(VOICE_TABLE_2000);
			for( i=0; i<m_RecordedSize; i++)
				m_pRecordBuf[i] = VOICE_TABLE_2000[i];
			break;
		case 1:
			m_RecordedSize = sizeof(VOICE_TABLE_DSTAR);
			for( i=0; i<sizeof(VOICE_TABLE_DSTAR); i++)
				m_pRecordBuf[i] = VOICE_TABLE_DSTAR[i];
			break;
		case 2:
			m_RecordedSize = sizeof(VOICE_TABLE_4800);
			for( i=0; i<m_RecordedSize; i++)
				m_pRecordBuf[i] = VOICE_TABLE_4800[i];
			break;
		case 3:
			m_RecordedSize = sizeof(VOICE_TABLE_9600);
			for( i=0; i<m_RecordedSize; i++)
				m_pRecordBuf[i] = VOICE_TABLE_9600[i];
			break;
	}
	m_Recording = FALSE;
	m_RecorderPos = 0;
}


void CSerialPort::SaveToFile()
{
int i;
int col;
FILE *stream = NULL;
	switch(m_VoiceRate)
	{
		case 0:
			stream = fopen( "c:\\voicedata2000.h", "wb" );	//Open file for write
			fprintf(stream, "const unsigned char VOICE_TABLE_2000[%d] = {\r\n",m_RecordedSize);
			break;
		case 1:
			stream = fopen( "c:\\voicedataDstar.h", "wb" );	//Open file for write
			fprintf(stream, "const unsigned char VOICE_TABLE_DSTAR[%d] = {\r\n",m_RecordedSize);
			break;
		case 2:
			stream = fopen( "c:\\voicedata4800.h", "wb" );	//Open file for write
			fprintf(stream, "const unsigned char VOICE_TABLE_4800[%d] = {\r\n",m_RecordedSize);
			break;
		case 3:
			stream = fopen( "c:\\voicedata9600.h", "wb" );	//Open file for write
			fprintf(stream, "const unsigned char VOICE_TABLE_9600[%d] = {\r\n",m_RecordedSize);
			break;
	}
	col=0;
	for(i=0; i<m_RecordedSize; i++)
	{
		fprintf(stream, "0x%2.2X, ", m_pRecordBuf[i]);
		col++;
		if(col >= 16)
		{
			col = 0;
			fprintf(stream, "\r\n");
		}
	}
	fprintf(stream, "\r\n};");
	if(stream)
		fclose( stream );
}



void CSerialPort::CalcBitErrors()
{
int i;
int p;
int b;
int j;
	for(i=0; i<24; i++)
		m_BitErrorMask[i] = 0;
	for(i=0; i<m_NumBitErrors; i++)
	{
		p = GetRandBitPos(m_NumFrameBytes*8);	//get a random bit position 0 to (m_NumFrameBytes*8)
		b = p%8;	//get bit position
		j = p/8;	//get byte position
		m_BitErrorMask[j] |= (1<<b);
	}
}

int CSerialPort::GetRandBitPos(int maxpos)
{
int r;
	do
	{
		r = rand()>>4;
		r &= 0x7F;
	}while( r>maxpos );
	return (r-1);
}

void CSerialPort::SaveToWaveFile(CString path)
{
WAVEFORMATEX wfx;
	m_Wave.OutClose();	//close any previous wave file if open
	if(m_RecordedSize>0)
	{
		wfx.wFormatTag = WAVE_FORMAT_PCM;
		wfx.nChannels = 1;
		wfx.wBitsPerSample = 16;
		wfx.nSamplesPerSec = 8000;
		wfx.nBlockAlign = 	wfx.nChannels *(wfx.wBitsPerSample/8);
		wfx.nAvgBytesPerSec = 	wfx.nSamplesPerSec * wfx.nBlockAlign;
		wfx.cbSize = 0;
		int error = m_Wave.OutOpen( &path, &wfx, 160*8, 100000000);
		if( !error )
			m_SaveArmed = TRUE;
		else
			m_SaveArmed = FALSE;
		m_Recording = FALSE;
		m_RecorderPos = 0;		//setting to 0 triggers playback from record buffer
	}
}
