// SerialPort.h: interface for the CSerialPort class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SERIALPORT_H__B3FE36C8_CAF3_4D28_8CC6_D387688A0A8E__INCLUDED_)
#define AFX_SERIALPORT_H__B3FE36C8_CAF3_4D28_8CC6_D387688A0A8E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SerialEx.h"
#include "ProtoColDefs.h"
#include "SerialEx.h"
#include "Ambe2020.h"
#include "SoundCard.h"
#include "wave.h"

/////////////////////////////////////////////////////////////////////////////
// horrible non-portable ASCP message buffer structure but....its fast and easy
//////////////////////////////////////////////////////////////////////////////
#define MAX_ASCP_BUFFER 8192
typedef union
{
	//Basic msg
	struct sasc1
	{
		unsigned __int8 h0;
		unsigned __int8 h1;
		unsigned __int8 parms[(MAX_ASCP_BUFFER-6)];
	}base;
	//Control Items
	struct sasc2
	{
		unsigned __int16 header;
		unsigned __int16 CItem;
		unsigned __int8 parms[(MAX_ASCP_BUFFER-4)];
	}citem;
	struct sasc3
	{
		unsigned __int16 header;
		unsigned __int16 CItem;
		unsigned __int16 parms[(MAX_ASCP_BUFFER-4)/2];
	}citemw;
	//Data Items
	struct sasc4
	{
		unsigned __int16 header;
		unsigned __int8 parms[(MAX_ASCP_BUFFER-2)];
	}Ditem8;
	struct sasc5
	{
		unsigned __int16 header;
		unsigned __int16 parms[(MAX_ASCP_BUFFER-2)/2];
	}Ditem16;
	unsigned __int8 buf[MAX_ASCP_BUFFER];
}ASCPstruct;

class CSerialPort : public CSerialEx  
{
public:
	void CannedVoice();
	CSerialPort();
	virtual ~CSerialPort();
	// Start the listener thread
	virtual LONG StartListener(int VoiceRate);		//over ride to do some custom initialization
	BOOL SendASCPMsg( ASCPstruct* pASCPmsg);
	void SetTouchTone(BOOL On, char digit );
	void SetRecordingState(BOOL On);
	void SetBitErrors(int num){ m_NumBitErrors = num;};
	int GetProgress();
	int GetBitErrors(){return m_RxBitErrors;};
	void SaveToWaveFile(CString path);

	ASCPstruct m_TxASCPmsg;
	BOOL m_VoiceDetect;
	CString m_SerialNumber;
protected:
	// Event handler
	virtual void OnEvent (EEvent eEvent, EError eError);
private:
	void CalcBitErrors();
	void AssembleMsg(unsigned __int8 ch);
	void ParseASCPmsg(ASCPstruct* pMsg);
	void SaveToFile();
	int GetRandBitPos(int maxpos);
	BOOL m_MsgActive;
	BOOL m_Recording;
	BOOL m_DTMFActive;
	BOOL m_SaveArmed;
	BOOL m_SaveActive;
	char m_BitErrorMask[24];
	int m_NumFrameBytes;
	int m_ParseState;
	int m_RxMsgLength;
	int m_RxMsgIndex;
	int m_MsgTimer;
	int m_RecordedSize;
	int m_RecorderPos;
	int m_VoiceRate;
	int m_NumBitErrors;
	int m_RxBitErrors;
	unsigned char* m_pRecordBuf;
	ASCPstruct m_RxASCPmsg;
	tOutFrame m_OutFrame;
	tInFrame m_InFrame;
	CWave m_Wave;
	CRITICAL_SECTION m_CriticalSection;	//for keeping threads from stomping on each other
};

#endif // !defined(AFX_SERIALPORT_H__B3FE36C8_CAF3_4D28_8CC6_D387688A0A8E__INCLUDED_)
