// SoundCard.cpp: implementation of the CSoundCard class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "math.h"
#include "AmbeTest.h"
#include "SoundCard.h"
#include "SerialPort.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSoundCard::CSoundCard()
{
int i;
	m_InDispPtr = 0;
	m_OutDispPtr = 0;
	for(i=0; i<MAX_DISPLAY_BUFSIZE ;i++)
	{
		m_InDisplayBuf[i] = 0;
		m_OutDisplayBuf[i] = 0;
	}
	m_ThreadsRunning = FALSE;
	m_pSndInThread = NULL;
	m_pSndOutThread = NULL;

}

CSoundCard::~CSoundCard()
{
	StopSoundCard();
}

int CSoundCard::StartSoundCard(int InCard, int OutCard)
{
int i,j;
	m_InBufHead = 0;
	m_InBufTail = 0;
	m_OutBufHead = 0;
	m_OutBufTail = 0;
	m_InCard = InCard;
	m_OutCard = OutCard;
	m_DeltaIn = 0;
	m_DeltaInSum = 0;
	m_DeltaInAve = 0;
	m_DeltaOut = 0;
	m_DeltaOutSum = 0;
	m_DeltaOutAve = 0;


	for(j=0; j<RINGBUF_SIZE; j++)
	{
		for (i=0; i<DATA_BUFSIZE; i++)
		{
			m_InBuf[j].Buf[i] = 0;
			m_OutBuf[j].Buf[i] = 0;
		}
	}
	if(m_pSndInThread)
		delete m_pSndInThread;
	//start worker thread 
	m_pSndInThread = AfxBeginThread(SndInThreadProc,
								this,
								THREAD_PRIORITY_NORMAL,
								0,
								CREATE_SUSPENDED);
	m_pSndInThread->m_bAutoDelete = FALSE;
	m_ThreadsRunning = TRUE;
	m_pSndInThread->ResumeThread();

	if(m_pSndOutThread)
		delete m_pSndOutThread;
	//start worker thread 
	m_pSndOutThread = AfxBeginThread(SndOutThreadProc,
								this,
								THREAD_PRIORITY_NORMAL,
								0,
								CREATE_SUSPENDED);
	m_pSndOutThread->m_bAutoDelete = FALSE;
	m_ThreadsRunning = TRUE;
	m_pSndOutThread->ResumeThread();
	return 0;
}

void CSoundCard::StopSoundCard()
{
DWORD eventstat;
	if( m_pSndInThread != NULL )		//if thread is already running
	{
		m_ThreadsRunning = FALSE;
		eventstat = ::WaitForSingleObject(m_pSndInThread->m_hThread, 1000);
		if(eventstat!=WAIT_OBJECT_0)
			eventstat = 0;
		if(m_pSndInThread)
			delete m_pSndInThread;
		m_pSndInThread = NULL;
	}
	if( m_pSndOutThread != NULL )		//if thread is already running
	{
		m_ThreadsRunning = FALSE;
		eventstat = ::WaitForSingleObject(m_pSndOutThread->m_hThread, 1000);
		if(eventstat!=WAIT_OBJECT_0)
			eventstat = 0;
		if(m_pSndOutThread)
			delete m_pSndOutThread;
		m_pSndOutThread = NULL;
	}
}

UINT CSoundCard::OpenOutput(int Card)
{
int i;
	m_OutDispPtr = 0;
	for(i=0; i<MAX_DISPLAY_BUFSIZE ;i++)
		m_OutDisplayBuf[i] = 0;
	m_OutWFX.wFormatTag = WAVE_FORMAT_PCM;
	m_OutWFX.nChannels = 1;
	m_OutWFX.wBitsPerSample = 16;
	m_OutWFX.nSamplesPerSec = 8000;
	m_OutWFX.nBlockAlign = 	m_OutWFX.nChannels *(m_OutWFX.wBitsPerSample/8);
	m_OutWFX.nAvgBytesPerSec = 	m_OutWFX.nSamplesPerSec * m_OutWFX.nBlockAlign;
	m_OutWFX.cbSize = 0;
	return m_Sound.OutOpen( &m_OutWFX, 160, 0, Card, TRUE);
}

UINT CSoundCard::OpenInput(int Card)
{
int i;
	m_InDispPtr = 0;
	for(i=0; i<MAX_DISPLAY_BUFSIZE ;i++)
		m_InDisplayBuf[i] = 0;
	m_InWFX.wFormatTag = WAVE_FORMAT_PCM;
	m_InWFX.nChannels = 1;
	m_InWFX.wBitsPerSample = 16;
	m_InWFX.nSamplesPerSec = 8000;
	m_InWFX.nBlockAlign = 	m_InWFX.nChannels *(m_InWFX.wBitsPerSample/8);
	m_InWFX.nAvgBytesPerSec = 	m_InWFX.nSamplesPerSec * m_InWFX.nBlockAlign;
	m_InWFX.cbSize = 0;
	return m_Sound.InOpen( &m_InWFX, 160, 0, Card);
}

void CSoundCard::Close()
{
	m_Sound.OutClose();
	m_Sound.InClose();
}



//////////////////////////////////////////////////////////////////////
// Worker Thread static procedures that just call the process loops
// needed to deal with C++ problems with threads
//////////////////////////////////////////////////////////////////////
UINT CSoundCard::SndInThreadProc(LPVOID pParam)
{
CSoundCard* pObject = (CSoundCard*)pParam;
	pObject->SndInThread();
	pObject->m_ThreadsRunning = FALSE;
	return 101;
}

//////////////////////////////////////////////////////////////////////
// Worker Thread static procedures that just call the process loops
// needed to deal with C++ problems with threads
//////////////////////////////////////////////////////////////////////
UINT CSoundCard::SndOutThreadProc(LPVOID pParam)
{
CSoundCard* pObject = (CSoundCard*)pParam;
	pObject->SndOutThread();
	pObject->m_ThreadsRunning = FALSE;
	return 102;
}

//////////////////////////////////////////////////////////////////////
//  Main processing loop for maintaining sound in data
//////////////////////////////////////////////////////////////////////
void CSoundCard::SndInThread()
{
SHORT tmp;
UINT Result;
	if( !m_Sound.IsInOpen() )
		Result = OpenInput(m_InCard);
	while(m_ThreadsRunning)	
	{
		int result = m_Sound.InRead(m_InBuf[m_InBufHead].Buf, DATA_BUFSIZE);	//will block on soundcard data
		if(result!=DATA_BUFSIZE)
			TRACE("\r\nReadError %d",result);
		for(int i=0; i<DATA_BUFSIZE; i++)
		{
			tmp = m_InBuf[m_InBufHead].Buf[i];
			m_InDisplayBuf[m_InDispPtr++] = tmp;
			if(m_InDispPtr >= MAX_DISPLAY_BUFSIZE) 
				m_InDispPtr = 0;
		}
		m_InBufHead++; 
		m_InBufHead &= (RINGBUF_SIZE-1);
		if(m_InBufHead == m_InBufTail)	//if overflowed Ring buffer
		{	//bump up halfway 
TRACE("\r\n Snd In Over" );
			m_InBufHead += (RINGBUF_SIZE/2);
			m_InBufHead &= (RINGBUF_SIZE-1);
			m_DeltaIn = 0;
		}
		m_DeltaIn++;
		m_DeltaInSum = m_DeltaInSum + m_DeltaIn - m_DeltaInAve;
		m_DeltaInAve = m_DeltaInSum>>4;

		Sleep(2);
	}
	m_Sound.InClose();
}

//////////////////////////////////////////////////////////////////////
//  Main processing loop for maintaining sound out data
//////////////////////////////////////////////////////////////////////
void CSoundCard::SndOutThread()
{
	if( !m_Sound.IsOutOpen() )
		OpenOutput(m_OutCard);
	while(m_ThreadsRunning)	
	{
		for(int i=0; i<DATA_BUFSIZE; i++)
		{
			m_OutDisplayBuf[m_OutDispPtr++] = m_OutBuf[m_OutBufTail].Buf[i];
			if(m_OutDispPtr >= MAX_DISPLAY_BUFSIZE) 
				m_OutDispPtr = 0;
		}
		int result = m_Sound.OutWrite(m_OutBuf[m_OutBufTail].Buf, DATA_BUFSIZE);
		m_OutBufTail++;
		m_OutBufTail &= (RINGBUF_SIZE-1);
		if(m_OutBufHead == m_OutBufTail)	//if underflowed Ring buffer
		{	//bump up halfway 
TRACE("\r\n Snd Out Under" );
			m_OutBufTail += (RINGBUF_SIZE/2);
			m_OutBufTail &= (RINGBUF_SIZE-1);
			m_DeltaOut = 0;
		}
		m_DeltaOut--;
		m_DeltaOutSum = m_DeltaOutSum + m_DeltaOut - m_DeltaOutAve;
		m_DeltaOutAve = m_DeltaOutSum>>4;
		Sleep(2);
	}
	m_Sound.OutClose();
}

void CSoundCard::PutSoundOutData( SHORT* pData )
{
	for(int i=0; i<DATA_BUFSIZE; i++)
		m_OutBuf[m_OutBufHead].Buf[i] = pData[i];
	m_OutBufHead++;
	m_OutBufHead &= (RINGBUF_SIZE-1);
	if(m_OutBufHead == m_OutBufTail)	//if overflowed Ring buffer
	{	//bump up halfway 
TRACE("\r\n Snd Out Over" );

		m_OutBufHead += (RINGBUF_SIZE/2);
		m_OutBufHead &= (RINGBUF_SIZE-1);
		m_DeltaOut = 0;
	}
	m_DeltaOut++;
}

void CSoundCard::GetSoundInData( SHORT* pData)
{
//static double phase = 0.0;
	for(int i=0; i<DATA_BUFSIZE; i++)
	{
		pData[i] = m_InBuf[m_InBufTail].Buf[i];
//		pData[i] = (short)(10000.0*cos(phase) );
//		phase += ((6.28*500.0)/8000.0 );
	}
	m_InBufTail++;
	m_InBufTail &= (RINGBUF_SIZE-1);
	if(m_InBufHead == m_InBufTail)	//if underflowed Ring buffer
	{	//bump up halfway 
TRACE("\r\n Snd In Under" );
		m_InBufTail += (RINGBUF_SIZE/2);
		m_InBufTail &= (RINGBUF_SIZE-1);
		m_DeltaIn = 0;
	}
	m_DeltaIn--;
}
